Lobby IoVM ImmutableSequence := ""
Lobby IoVM String := ImmutableSequence

Sequence oldStringSlots := Map clone do(
atPut("upper", "asUppercase")
atPut("lower", "asLowercase")
atPut("appendPath", "Path with(,)")
atPut("upper", "asUppercase")
atPut("removeFromTo", "removeSlice")
atPut("insert", "atInsertSeq(index, seq)")
atPut("fromTo", "slice")
atPut("substring", "slice")
atPut("beginsWith", "beginsWithSeq")
atPut("endsWith", "endsWithSeq")
atPut("after", "afterSeq")
atPut("before", "beforeSeq")
atPut("capitalized", "asCapitalized")
atPut("containsAnyCase", "containsAnyCaseSeq")
atPut("contains", "containsSeq")
atPut("swapIndexWithIndex", "swapIndices")
atPut("random", "anyOne")
atPut("randomize", "shuffle")
atPut("removeLastPathComponent", "pathComponent")
atPut("centered", "[no longer supported]")
atPut("leftJustified", "[no longer supported]")
atPut("rightJustified", "[no longer supported]")
atPut("length", "size")
atPut("find", "findSeq")
)

Sequence forward := method(
	slotName := call message name
	newSlot := oldStringSlots at(slotName)
	if (newSlot, Exception raise("Sequence does not respond to " .. slotName .. " (this method has been replaced by " .. newSlot .. ")"))
	Exception raise("Sequence does not respond to " .. slotName)
)

Sequence repeated := method(n,
	s := Sequence clone
	for(i, 0, n, s appendSeq(self))
	s 
)

Sequence setSlot("..", method(arg, self asString cloneAppendSeq(arg asString)))


Sequence do(
    docSlot("alignLeftInPlace(width, [padding])", """
    """)
    alignLeftInPlace := method(width, padding,
        originalSize := size
        padding = padding ifNilEval(" ")
        ((width - size) / padding size) ceil repeatTimes(appendSeq(padding))
        setSize(width max(originalSize))
    )

    docSlot("alignLeft(width, [padding])", """
    Example:
    <pre>
    Io> "abc" alignLeft(10, "-")
    ==> abc-------
    Io> "abc" alignLeft(10, "-=")
    ==> abc-=-=-=-
    </pre>
    """)
    alignLeft := method(width, padding,
        asMutable alignLeftInPlace(width, padding)
    )

    docSlot("alignRight(width, [padding]", """
    Example:
    <pre>
    Io> "abc" alignRight(10, "-")
    ==> -------abc
    Io> "abc" alignRight(10, "-=")
    ==> -=-=-=-abc
    </pre>
    """)
    alignRight := method(width, padding,
        Sequence clone alignLeftInPlace(width - size, padding) appendSeq(self)
    )

    docSlot("alignCenter(width, [padding]", """
    Example:
    <pre>
    Io> "abc" alignCenter(10, "-")
    ==> ---abc----
    Io> "abc" alignCenter(10, "-=")
    ==> -=-abc-=-=
    </pre>
    """)
    alignCenter := method(width, padding,
        alignRight(((size + width) / 2) floor, padding) alignLeftInPlace(width, padding)
    )
    
    asSimpleString := method("\"" .. self asString .. "\"")
)

Sequence splitNoEmpties := method(
    docSlot("split(optionalArg1, optionalArg2, ...)", """
    Returns a list containing the non-empty sub-sequences of the receiver divided by the given arguments.
    If no arguments are given the sequence is split on white space.
    Examples:
    <pre>
    "a   b  c d" splitNoEmpties => list("a", "b", "c", "d")
    "a***b**c*d" splitNoEmpties("*") => list("a", "b", "c", "d")
    "a***b||c,d" splitNoEmpties("*", "|", ",") => list("a", "b", "c", "d")
    </pre>
    """)

    self doMessage(
        call message clone setName("split") setAttachedMessage(nil) setNextMessage(nil)
    ,
        call sender
    ) selectInPlace(s, s size != 0)
)
