docSlot("getOptions(args)", """
	This primitive is used to get command line options similar to Cs getopt(). 
	It returns a map in containing the left side of the argument, with the 
	value of the right side. (The key will not contain
	the beginning dashes (--).
 	<p>
	Example:
 	<pre>
	options := System getOptions(args)
	options foreach(k, v,
	  if(v type == List type,
	    v foreach(i, j, writeln("Got unnamed argument with value: " .. j))
	    continue
	  )
	  writeln("Got option: " .. k .. " with value: " .. v)
	)
 	</pre>
""")


System do(
  getOptions := method(arguments,
    opts := Map clone
    optname := Sequence clone
    optvalue := Sequence clone
    optsNoKey := List clone
  
    arguments foreach(i, arg,
      if(arg beginsWithSeq("--") isNil) then(
        optsNoKey append(arg)
        continue
      )
      if(arg containsSeq("=")) then(
        optname = arg clone asMutable clipAfterStartOfSeq("=") clipBeforeEndOfSeq("--")
        optvalue = arg clone asMutable clipBeforeEndOfSeq("=")
      ) else(
        optname = arg clone asMutable clipBeforeEndOfSeq("--")
        optvalue = ""
      )
      opts atPut(optname, optvalue)
    )
    if(optsNoKey last != nil, opts atPut("", optsNoKey))
    opts
  )
)

